from django.conf import settings

ROOT = getattr(settings, 'CROPPER_ROOT', 'cropped/').rstrip('/')
MAX_WIDTH  = getattr(settings, 'CROPPER_MAX_WIDTH',  1680)
MAX_HEIGHT = getattr(settings, 'CROPPER_MAX_HEIGHT', 1024)
ALLOWED_DIMENSIONS = getattr(settings, 'CROPPER_ALLOWED_DIMENSIONS', ())

SHOW_ADMIN = getattr(settings, 'CROPPER_SHOW_ADMIN', True)



from django import test
from django.conf import settings
from django.core.urlresolvers import reverse
from cropper.models import Original, Cropped
from django.utils import simplejson
from django.core.files.base import ContentFile
import os
import filecmp

CROP_X = 304
CROP_Y = 151
CROP_W = 175
CROP_H = 193

MAX_WIDTH = 1680
MAX_HEIGHT = 1024

def get_filename(name='original.jpg'):
    """
    Returns the full path to test images
    """
    return os.path.join(os.path.dirname(__file__), name)


class BaseTestCase(test.TestCase):
    urls = 'cropper.urls'

    def setUp(self):
        self.client = test.Client()
        self.url = reverse('cropper_upload')


class ModelTest(test.TestCase):
    def create_original(self):
        original = Original()
        original.image.save(
            os.path.basename(self.filename),
            ContentFile(open(self.filename).read())
        )
        return original

    def setUp(self):
        self.filename = get_filename()
        self.original = self.create_original()

    def tearDown(self):
        settings.CROPPER_MAX_WIDTH  = MAX_WIDTH
        settings.CROPPER_MAX_HEIGHT = MAX_HEIGHT

    def test_original_max_width_exceed(self):
        settings.CROPPER_MAX_WIDTH = 30
        self.original = self.create_original()

    def test_original(self):

        self.assertTrue(filecmp.cmp(self.filename, self.original.image.path))

    def test_cropped(self):
        cropped = Cropped.objects.create(original = self.original,
            w = CROP_W,
            h = CROP_H,
            x = CROP_X,
            y = CROP_Y)
        self.assertTrue(filecmp.cmp(get_filename('cropped_304-151-175-193.jpg'), cropped.image.path))


class UploadTestCase(BaseTestCase):
    def upload_scenario(self):
        page = self.client.get(self.url)

        self.assertEquals(200, page.status_code)
        self.assertIn('form', page.context)
        self.assertIn('multipart/form-data', page.content)
        self.assertIn('submit', page.content)

        page = self.client.post(self.url, data={
            'image': open(get_filename())
        }, follow=True)

        self.assertEquals(200, page.status_code)
        self.assertIn('form', page.context)
        self.assertIn('original', page.context)
        return page.context['original']

    def test_crop_without_ahax(self):
        original = self.upload_scenario()
        page = self.client.post(original.get_absolute_url(), data={
            'original'  : original.pk,
            'x'         : CROP_X,
            'y'         : CROP_Y,
            'w'         : CROP_W,
            'h'         : CROP_H
        })
        self.assertIn('original', page.context)
        self.assertIn('cropped', page.context)
        self.assertTrue(filecmp.cmp(get_filename('cropped_304-151-175-193.jpg'), page.context['cropped'].image.path))

    def test_ajax_crop(self):
        original = self.upload_scenario()
        page = self.client.post(original.get_absolute_url(), data={
            'original'  : original.pk,
            'x'         : CROP_X,
            'y'         : CROP_Y,
            'w'         : CROP_W,
            'h'         : CROP_H
        }, HTTP_X_REQUESTED_WITH='XMLHttpRequest')

        json = simplejson.loads(page.content)

        self.assertIn('image',      json)
        self.assertIn('url',        json['image'])
        self.assertIn('width',      json['image'])
        self.assertIn('height',     json['image'])
        self.assertEquals(CROP_W,   json['image']['width'])
        self.assertEquals(CROP_H,   json['image']['height'])


from django.contrib import admin
from cropper.models import Original, Cropped
from cropper import settings

class OriginalAdmin(admin.ModelAdmin):
    """
    Admin class for original image model
    """

class CroppedAdmin(admin.ModelAdmin):
    """
    Admin class for cropped image model
    """

if settings.SHOW_ADMIN:
    admin.site.register(Original, OriginalAdmin)
    admin.site.register(Cropped, CroppedAdmin)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Deleting field 'Original.name'
        db.delete_column('cropper_original', 'name')

        # Deleting field 'Cropped.name'
        db.delete_column('cropper_cropped', 'name')


    def backwards(self, orm):
        
        # Adding field 'Original.name'
        db.add_column('cropper_original', 'name', self.gf('django.db.models.fields.CharField')(default='', max_length=255), keep_default=False)

        # Adding field 'Cropped.name'
        db.add_column('cropper_cropped', 'name', self.gf('django.db.models.fields.CharField')(default='', max_length=255), keep_default=False)


    models = {
        'cropper.cropped': {
            'Meta': {'object_name': 'Cropped'},
            'h': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'h_display': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'original': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'cropped'", 'to': "orm['cropper.Original']"}),
            'w': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'w_display': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'x': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'y': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'})
        },
        'cropper.original': {
            'Meta': {'object_name': 'Original'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'image_height': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'image_width': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'})
        }
    }

    complete_apps = ['cropper']



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError
from django.conf import settings as django_settings
from cropper import settings
import Image
import os
import uuid

def dimension_validator(image):
    """
    """
    if settings.MAX_WIDTH != 0 and image.width > settings.MAX_WIDTH:
        raise ValidationError(_('Image width greater then allowed'))

    if settings.MAX_HEIGHT != 0 and image.height > settings.MAX_HEIGHT:
        raise ValidationError(_('Image height greater then allowed'))


class Original(models.Model):
    def upload_image(self, filename):
        return u'{path}/{name}.{ext}'.format(path=settings.ROOT,
                                             name=uuid.uuid4().hex,
                                             ext=os.path.splitext(filename)[1].strip('.'))

    def __unicode__(self):
        return unicode(self.image)

    @models.permalink
    def get_absolute_url(self):
        return 'cropper_crop', [self.pk]    

    image = models.ImageField(_('Original image'),
                            upload_to=upload_image,
                            width_field='image_width',
                            height_field='image_height',
                            validators=[dimension_validator])
    image_width = models.PositiveIntegerField(_('Image width'),
                                                editable=False,
                                                default=0)
    image_height = models.PositiveIntegerField(_('Image height'),
                                               editable=False,
                                               default=0)


class Cropped(models.Model):
    def __unicode__(self):
        return u'%s-%sx%s' % (self.original, self.w, self.h)

    def upload_image(self, filename):
        return '%s/crop-%s' % (settings.ROOT, filename)

    def save(self, *args, **kwargs):
        source = self.original.image.path
        target = self.upload_image(os.path.basename(source))

        Image.open(source).crop([
            self.x,             # Left
            self.y,             # Top
            self.x + self.w,    # Right
            self.y + self.h     # Bottom
        ]).save(django_settings.MEDIA_ROOT + os.sep + target)

        self.image = target        
        super(Cropped, self).save(*args, **kwargs)

    original = models.ForeignKey(Original,
                                 related_name='cropped',
                                 verbose_name=_('Original image'))
    image = models.ImageField(_('Image'),
                              upload_to=upload_image,
                              editable=False)
    x = models.PositiveIntegerField(_('offset X'),
                                   default=0)
    y = models.PositiveIntegerField(_('offset Y'),
                                   default=0)
    w = models.PositiveIntegerField(_('cropped area width'),
                                    blank=True,
                                    null=True)
    h = models.PositiveIntegerField(_('cropped area height'),
                                    blank=True,
                                    null=True)

    class Meta(object):
        verbose_name = _('cropped image')
        verbose_name_plural = _('cropped images')




from django.http import HttpResponse
from django.shortcuts import get_object_or_404, render, redirect
from django.views.generic.edit import FormView
from cropper import settings
from cropper.models import Original
from cropper.forms import CroppedForm, OriginalForm
import json


class UploadView(FormView):
    """
    Upload picture to future cropping
    """
    form_class = OriginalForm
    template_name = 'cropper/upload.html'

    def success(self, request, form, original):
        return redirect(original)

    def form_valid(self, form):
        original = form.save()
        return self.success(self.request, form, original)


class CropView(FormView):
    """
    Crop picture and save result into model
    """
    form_class = CroppedForm
    template_name = 'cropper/crop.html'

    def get_object(self):
        """
        Returns the original image object
        """
        return get_object_or_404(Original, pk=self.kwargs['original_id'])

    def get_initial(self):
        """
        Initial dictionary that passed into form instance arguments
        """
        return {'original': self.get_object()}

    def get_context_data(self, **kwargs):
        """
        Context dictionary that passed into template renderer
        """
        return {
            'form': self.get_form(self.form_class),
            'original': self.get_object(),
            'cropped': None
        }

    def form_valid(self, form):
        cropped = form.save(commit=False)
        cropped.save()

        return self.success(request=self.request,
                            form=form,
                            original=self.get_object(),
                            cropped=cropped)

    def success(self, request, form, original, cropped):
        """
        Default success crop handler
        """
        return HttpResponse(json.dumps({'image': {'url': cropped.image.url,
                                                  'width': cropped.w,
                                                  'height': cropped.h,
            }}), mimetype='application/x-json') if request.is_ajax() else \
            render(request, 'cropper/crop.html', {'form': form,
                                                  'cropped': cropped,
                                                  'original': original
            })



from django.conf.urls import url
from cropper.views import UploadView, CropView

urlpatterns = [
    url('^$', UploadView.as_view(), name='cropper_upload'),
    url('^(?P<original_id>\d+)/$', CropView.as_view(),   name='cropper_crop'),
]



from django import forms
from django.utils.translation import ugettext_lazy as _
from cropper.models import Cropped, Original


class OriginalForm(forms.ModelForm):
    """
    Form class for upload images will be cropped
    """
    class Meta(object):
        model = Original


class CroppedForm(forms.ModelForm):
    """
    Form class for crop images
    """

    def _dimension_clean(self, field, key, offset='not_exists'):
        """
        Form helper. Validates for cropped area offset left, offset top,
        area width and  height values.
        """
        value  = self.cleaned_data.get(key, 0)
        offset  = self.cleaned_data.get(offset, 0)
        original = self.cleaned_data.get('original')

        if not original:
            return value

        if value + offset > getattr(original, 'image_%s' % field):
            raise forms.ValidationError(_('Value exceeds picture dimension'))

        return value

    def clean_x(self):
        """
        Validates for cropped area offset left
        """
        return self._dimension_clean('width', 'x')

    def clean_y(self):
        """
        Validates for cropped area offset top
        """
        return self._dimension_clean('height', 'y')

    def clean_w(self):
        """
        Validates for cropped area width
        """
        return self._dimension_clean('width', 'w', 'x')

    def clean_h(self):
        """
        Validates for cropped area height
        """
        return self._dimension_clean('height', 'h', 'y')

    class Meta(object):
        model = Cropped
        widgets = {
            'original': forms.HiddenInput(),
        }




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import setuptools
import os

version='0.3.0'
package = 'cropper'

path = os.path.join(os.path.dirname(__file__), 'cropper')
setuptools.setup(
    name = 'django-image-cropper',
    version = version,
    author  = 'marazmiki',
    author_email = 'marazmiki@gmail.com',
    url = 'http://bitbucket.org/marazmiki/django-image-cropper/',
    download_url = 'http://bitbucket.org/marazmiki/django-image-cropper/get/tip.zip',

    description = 'This app allows upload and crop images',
    long_description = open('README.rst').read(),
    license = 'MIT license',
    install_requires = ['django>=1.5', 'Pillow'],
    packages = setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'Original'
        db.create_table('cropper_original', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('image', self.gf('django.db.models.fields.files.ImageField')(max_length=100)),
            ('image_width', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
            ('image_height', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
        ))
        db.send_create_signal('cropper', ['Original'])

        # Adding model 'Cropped'
        db.create_table('cropper_cropped', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('original', self.gf('django.db.models.fields.related.ForeignKey')(related_name='cropped', to=orm['cropper.Original'])),
            ('image', self.gf('django.db.models.fields.files.ImageField')(max_length=100)),
            ('x', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
            ('y', self.gf('django.db.models.fields.PositiveIntegerField')(default=0)),
            ('w', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True)),
            ('h', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True)),
            ('w_display', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True)),
            ('h_display', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True)),
        ))
        db.send_create_signal('cropper', ['Cropped'])


    def backwards(self, orm):
        
        # Deleting model 'Original'
        db.delete_table('cropper_original')

        # Deleting model 'Cropped'
        db.delete_table('cropper_cropped')


    models = {
        'cropper.cropped': {
            'Meta': {'object_name': 'Cropped'},
            'h': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'h_display': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'original': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'cropped'", 'to': "orm['cropper.Original']"}),
            'w': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'w_display': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'x': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'y': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'})
        },
        'cropper.original': {
            'Meta': {'object_name': 'Original'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'image_height': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'image_width': ('django.db.models.fields.PositiveIntegerField', [], {'default': '0'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        }
    }

    complete_apps = ['cropper']
